#!/usr/bin/env python
#-*- coding: utf-8 -*-
"""
This program generates waveform composed of 3 damped sine waves with similar frequency

Filip Dominec, dominecf@fzu.cz
"""

import numpy as np
from scipy.constants import c, hbar, pi
import matplotlib
matplotlib.rc('text', usetex=True)
matplotlib.rc('text.latex', preamble = \
        '\usepackage{amsmath}, \usepackage{yfonts}, \usepackage{txfonts}, \usepackage{lmodern},')
import matplotlib.pyplot as plt

## Time-domain data 
x = np.linspace(0, 5, 200)
frequencies = [4., 4.5, 4.8]
y = np.sin(2*np.pi*x*4)/np.exp(x*.5)*.1 + np.sin(2*np.pi*x*4.5)/np.exp(x*.5)*3  + np.sin(2*np.pi*x*4.8)/np.exp(x*1.5)
dt = x[1]-x[0]
plt.subplot(211)

plt.plot(x, y, color="#FF8800", label=u"$y'$", ls='-')      # (optional) plot amplitude

with open('hitest.dat', 'w') as outfile: 
    outfile.write("#t[s]\t E(t)\n")
    np.savetxt(outfile, zip(x/2, y), fmt="%.8e")
plt.ylabel(u"signal"); 
plt.xlabel(u"time"); 

## Fourier transform
plt.subplot(212)
freq = np.fft.fftshift(np.fft.fftfreq(len(x), d=(x[1]-x[0])))         # calculate the frequency axis with proper spacing
yf = np.fft.fftshift(np.fft.fft(y)) / len(x) * 2*np.pi  # calculate the FFT values, normalize to number of data points
truncated = np.logical_and(freq>-10., freq<10.)         # (optional) get the frequency range
(yf, freq) = map(lambda x: x[truncated], (yf, freq))    # (optional) truncate the data points
plt.plot(freq, np.abs(yf), color="#0088FF", label=u"FFT", ls='-')      # (optional) plot amplitude
for f in frequencies:
    plt.axvspan(f, f, color='r', lw=.5)
    plt.axvspan(-f, -f, color='r', lw=.5)
plt.plot([], [], color="r", label=u"oscillators", ls='-')      # (optional) plot amplitude

def find_maxima(x, y, minimum_value=.1):
    """ 
    Returns the x points where 
    1) y has a local maximum (i. e. dx/dy goes negative) AND 
    2) where y is above minimum_value 
    """
    d = y[1:-1] - y[0:-2]   ## naïve first derivative
    maxima = x[1:][np.sign(d[0:-2])-np.sign(d[1:-1]) + np.sign(y[2:-2]-minimum_value)==3]
    return maxima 
maxfreq =  find_maxima(freq, abs(yf), minimum_value=.0)
maxyf =  np.interp(maxfreq, freq, abs(yf))
print "FFT maxima: frequencies, values:", maxfreq, maxyf
plt.plot(maxfreq, maxyf, color="#8800FF", label=u"FFT maxima", ls='', marker='^')      # (optional) plot amplitude

## Harminv
import subprocess
dt = x[1]-x[0]
subprocess.Popen('harminv 0-1000 -t %g -d 1 -f 30 < hitest.dat > hiout.dat' % dt, shell=True, stdout=subprocess.PIPE).stdout.read().strip()

(mf, md, mQ, mA, mp, merr) = np.loadtxt('hiout.dat', usecols=list(range(6)), unpack=True, delimiter=', ', skiprows=1)
print mf*2, md
plt.plot(mf*2, mA, color="#FF8800", label=u"harminv", ls='', marker='*')      # (optional) plot amplitude


## Finish the graph + save 
plt.ylabel(u"spectral amplitude"); 
plt.xlabel(u"frequency"); 
#plt.ylim((-0.1,1.1)); plt.xscale('linear')
#plt.xlim((-0.1,1.1)); plt.xscale('linear')
plt.grid()
plt.legend(prop={'size':10}, loc='upper right')
plt.savefig("output.png", bbox_inches='tight')

